@@ -1,7 +1,7 @@
 
 /*
  *  agUtils.c
- *  $Id: agUtils.c,v 4.5 2005/06/07 22:26:43 bkorb Exp $
+ *  $Id: agUtils.c,v 4.7 2006/01/28 21:26:40 bkorb Exp $
  *  This is the main routine for autogen.
  */
 
@@ -115,6 +115,9 @@ doOptions( int arg_ct, char** arg_vec )
         }
     }
 
+    if (! HAVE_OPT( TIMEOUT ))
+        OPT_VALUE_TIMEOUT = AG_DEFAULT_TIMEOUT;
+
     /*
      *  IF the definitions file has been disabled,
      *  THEN a template *must* have been specified.
@@ -198,7 +201,7 @@ doOptions( int arg_ct, char** arg_vec )
 
 
 LOCAL tCC*
-getDefine( tCC* pzDefName )
+getDefine( tCC* pzDefName, ag_bool check_env )
 {
     tCC**   ppz;
     int     ct;
@@ -222,7 +225,7 @@ getDefine( tCC* pzDefName )
                 return (pzEq != NULL) ? pzEq+1 : "";
         }
     }
-    return getenv( pzDefName );
+    return check_env ? getenv( pzDefName ) : NULL;
 }
 
 
