@@ -1,7 +1,7 @@
 
 /*
  *  autogen.h
- *  $Id: autogen.h,v 4.11 2005/12/05 20:51:36 bkorb Exp $
+ *  $Id: autogen.h,v 4.12 2006/01/24 23:22:16 bkorb Exp $
  *  Global header file for AutoGen
  */
 
@@ -191,12 +191,11 @@ struct macro_desc {
 
 struct template_desc {
     tTlibMark   magic;       /* TEMPLATE_MAGIC_MARKER    */
-    int         fd;          /* mmap file descriptor     */
     size_t      descSize;    /* Structure Size           */
     char*       pNext;       /* Next Pointer             */
     int         macroCt;     /* Count of Macros          */
-    tCC*        pzFileName;  /* Name of Macro File       */
-    char*       pzTplName;   /* Template Name Pointer    */
+    tCC*        pzTplFile;   /* Name of template file    */
+    char*       pzTplName;   /* Defined Macro Name       */
     char*       pzTemplText; /* offset of the text       */
     char        zStartMac[MAX_SUFFIX_LEN];
     char        zEndMac[MAX_SUFFIX_LEN];
@@ -424,9 +423,6 @@ MKSTRING( ShellEnv,   "SHELL" );
 MKSTRING( Nil,        "" );
 MKSTRING( DefaultNil, "" );
 
-extern void unloadTemplate( tTemplate* pT );
-extern void unloadDefs( void );
-
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
  *
  *  GLOBAL PROCEDURES
@@ -437,16 +433,8 @@ extern size_t strlcpy( char* dest, const char* src, size_t n );
 #ifdef ENABLE_FMEMOPEN
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
  *
- *  IF we have fopencookie or funopen, then we also have our own fmemopen...
- *
- *  Special ioctl/"seek" requests
+ *  IF we have fopencookie or funopen, then we also have our own fmemopen
  */
-#define FMEM_IOCTL_BUF_ADDR  0x80000001
-
-/*
- *  Save the buffer on close
- */
-#define FMEM_IOCTL_SAVE_BUF  0x80000002
 
 extern FILE * ag_fmemopen (void *buf, ssize_t len, const char *mode);
 #endif
@@ -477,6 +465,12 @@ extern void  ag_scmStrings_deinit( void );
 extern void  ag_scmStrings_free(   void );
 extern char* ag_scribble( size_t size );
 
+#if defined(DEBUG_ENABLED)
+  extern void manageAllocatedData( void* pd );
+#else
+# define manageAllocatedData(_ptr)
+#endif
+
 /*
  *  Code variations based on the version of Guile:
  */
@@ -582,7 +576,7 @@ static inline SCM ag_eval( tCC* pzStr )
     pzLastScheme = pzStr;
 
     res = ag_scm_c_eval_string_from_file_line(
-        pzStr, pCurTemplate->pzFileName, pCurMacro->lineNo );
+        pzStr, pCurTemplate->pzTplFile, pCurMacro->lineNo );
 
     pzLastScheme = pzSaveScheme;
     return res;
