@@ -1,5 +1,5 @@
 /*
- *  $Id: defFind.c,v 4.4 2005/06/07 22:26:43 bkorb Exp $
+ *  $Id: defFind.c,v 4.7 2006/01/24 23:22:16 bkorb Exp $
  *
  *  This module locates definitions.
  */
@@ -34,13 +34,13 @@ typedef struct defEntryList tDefEntryList;
 
 tSCC zNameRef[]   = "Ill formed name ``%s'' in %s line %d\n";
 
-tSC zDefinitionName[ MAXPATHLEN ];
+static char zDefinitionName[ MAXPATHLEN ];
 
 static tDefEntry* findEntryByIndex( tDefEntry* pE, char* pzScan );
 
 #define ILLFORMEDNAME() \
     AG_ABEND( aprf( zNameRef, zDefinitionName, \
-              pCurTemplate->pzFileName, pCurMacro->lineNo ));
+              pCurTemplate->pzTplFile, pCurMacro->lineNo ));
 
 
 /* = = = START-STATIC-FORWARD = = = */
@@ -120,7 +120,7 @@ findEntryByIndex( tDefEntry* pE, char* pzScan )
         {
             char  svch = *pzScan;
             *pzScan = NUL;
-            pzVal    = getDefine( pzDef );
+            pzVal   = getDefine( pzDef, AG_TRUE );
             *pzScan = svch;
         }
 
@@ -187,7 +187,7 @@ badName( char* pzD, const char* pzS, size_t srcLen )
     memcpy( (void*)pzD, (void*)pzS, srcLen );
     pzD[ srcLen ] = NUL;
     fprintf( pfTrace, zNameRef, pzD,
-             pCurTemplate->pzFileName, pCurMacro->lineNo );
+             pCurTemplate->pzTplFile, pCurMacro->lineNo );
     return srcLen + 1;
 }
 
@@ -421,7 +421,7 @@ defEntrySearch( char* pzName, tDefCtx* pDefCtx, ag_bool* pIsIndexed )
     breakCh  = *pcBrace;
     *pcBrace = NUL;
 
-    *pIsIndexed |= (breakCh == '[');
+    if (breakCh == '[') *pIsIndexed = AG_TRUE;
 
     for (;;) {
         /*
