@@ -1,6 +1,6 @@
 
 /*
- *  $Id: expOutput.c,v 4.5 2005/06/07 22:26:43 bkorb Exp $
+ *  $Id: expOutput.c,v 4.8 2006/01/24 23:22:16 bkorb Exp $
  *
  *  This module implements the output file manipulation function
  */
@@ -81,12 +81,21 @@ removeWriteAccess( int fd )
 static void
 addWriteAccess( char* pzFileName )
 {
-    struct stat    sbuf;
+    struct stat sbuf;
+
+#ifdef DEBUG_ENABLED
+    /*
+     *  "stat(2)" does not initialize the entire structure.
+     */
+    memset( &sbuf, NUL, sizeof(sbuf) );
+#endif
+
     stat( pzFileName, &sbuf );
+
     /*
      *  Or in the user write bit
      */
-    sbuf.st_mode = sbuf.st_mode | S_IWUSR;
+    sbuf.st_mode |= S_IWUSR;
     chmod( pzFileName, sbuf.st_mode & S_IAMB );
 }
 
@@ -129,9 +138,9 @@ ag_scm_out_delete( void )
 SCM
 ag_scm_out_move( SCM new_file )
 {
-    size_t sz = SCM_LENGTH( new_file );
+    size_t sz = AG_SCM_STRLEN( new_file );
     char*  pz = (char*)AGALOC( sz + 1, "new file name string" );
-    memcpy( pz, SCM_CHARS( new_file ), sz );
+    memcpy( pz, AG_SCM_CHARS( new_file ), sz );
     pz[ sz ] = NUL;
 
     rename( pCurFp->pzOutName, pz );
@@ -164,14 +173,16 @@ ag_scm_out_pop( SCM ret_contents )
     if (pCurFp->pPrev == NULL)
         AG_ABEND( "ERROR:  Cannot pop output with no output pushed\n" );
 
-    if (gh_boolean_p( ret_contents ) && SCM_NFALSEP( ret_contents )) {
+    if (AG_SCM_BOOL_P( ret_contents ) && SCM_NFALSEP( ret_contents )) {
         long pos = ftell( pCurFp->pFile );
-        res = scm_makstr( pos, 0 );
+
         if (pos != 0) {
+            char* pz = ag_scribble( pos );
             rewind( pCurFp->pFile );
-            if (fread( SCM_CHARS( res ), pos, 1, pCurFp->pFile ) != 1)
+            if (fread( pz, pos, 1, pCurFp->pFile ) != 1)
                 AG_ABEND( aprf( "Error %d (%s) rereading output\n",
                                 errno, strerror( errno )));
+            res = AG_SCM_STR2SCM( pz, pos );
         }
     }
 
@@ -270,13 +281,13 @@ ag_scm_out_push_add( SCM new_file )
     tFpStack* p;
     char*  pzNewFile;
 
-    if (! gh_string_p( new_file ))
+    if (! AG_SCM_STRING_P( new_file ))
         return SCM_UNDEFINED;
 
     {
-        size_t sz = SCM_LENGTH( new_file );
+        size_t sz = AG_SCM_STRLEN( new_file );
         pzNewFile = (char*)AGALOC( sz + 1, "append file name string" );
-        memcpy( pzNewFile, SCM_CHARS( new_file ), sz );
+        memcpy( pzNewFile, AG_SCM_CHARS( new_file ), sz );
         pzNewFile[ sz ] = NUL;
     }
 
@@ -324,10 +335,10 @@ ag_scm_out_push_new( SCM new_file )
     p->pPrev = pCurFp;
     p->flags = FPF_FREE;
 
-    if (gh_string_p( new_file )) {
-        size_t sz = SCM_LENGTH( new_file );
+    if (AG_SCM_STRING_P( new_file )) {
+        size_t sz = AG_SCM_STRLEN( new_file );
         pzNewFile = (char*)AGALOC( sz + 1, "new file name string" );
-        memcpy( pzNewFile, SCM_CHARS( new_file ), sz );
+        memcpy( pzNewFile, AG_SCM_CHARS( new_file ), sz );
         pzNewFile[ sz ] = NUL;
         unlink( pzNewFile );
         addWriteAccess( pzNewFile );
@@ -357,6 +368,7 @@ ag_scm_out_push_new( SCM new_file )
             }
 
             pzTemp = aprf( "%s/agtmp.XXXXXX", pz );
+            manageAllocatedData((void*)pzTemp);
             TAGMEM( pzTemp, "Saved temp file template" );
         }
 
@@ -412,12 +424,12 @@ ag_scm_out_switch( SCM new_file )
     struct utimbuf tbuf;
     char*  pzNewFile;
 
-    if (! gh_string_p( new_file ))
+    if (! AG_SCM_STRING_P( new_file ))
         return SCM_UNDEFINED;
     {
-        size_t sz = SCM_LENGTH( new_file );
+        size_t sz = AG_SCM_STRLEN( new_file );
         pzNewFile = (char*)AGALOC( sz + 1, "new file name string" );
-        memcpy( pzNewFile, SCM_CHARS( new_file ), sz );
+        memcpy( pzNewFile, AG_SCM_CHARS( new_file ), sz );
         pzNewFile[ sz ] = NUL;
     }
 
@@ -463,7 +475,7 @@ ag_scm_out_switch( SCM new_file )
 SCM
 ag_scm_out_depth( void )
 {
-    return gh_int2scm( outputDepth );
+    return AG_SCM_INT2SCM( outputDepth );
 }
 
 
@@ -480,7 +492,7 @@ ag_scm_out_name( void )
 {
     tFpStack* p = pCurFp;
     while (p->flags & FPF_UNLINK)  p = p->pPrev;
-    return gh_str02scm( (void*)p->pzOutName );
+    return AG_SCM_STR02SCM( (void*)p->pzOutName );
 }
 
 
@@ -514,7 +526,7 @@ ag_scm_out_line( void )
         fseek( pCurFp->pFile, svpos, SEEK_SET );
     } while(0);
 
-    return gh_int2scm( lineNum );
+    return AG_SCM_INT2SCM( lineNum );
 }
 
 /*
