@@ -1,6 +1,6 @@
 
 /*
- *  $Id: funcCase.c,v 4.3 2005/01/24 00:00:32 bkorb Exp $
+ *  $Id: funcCase.c,v 4.7 2006/01/24 23:22:16 bkorb Exp $
  *
  *  This module implements the CASE text function.
  */
@@ -21,8 +21,8 @@
  *  You should have received a copy of the GNU General Public License
  *  along with AutoGen.  See the file "COPYING".  If not,
  *  write to:  The Free Software Foundation, Inc.,
- *             59 Temple Place - Suite 330,
- *             Boston,  MA  02111-1307, USA.
+ *             51 Franklin Street, Fifth Floor,
+ *             Boston, MA  02110-1301, USA.
  */
 
 #undef  IS_LOW
@@ -443,7 +443,7 @@ ag_scm_string_eqv_p( SCM text, SCM substr )
      *  We are overloading the "=" operator.  Our arguments may be
      *  numbers...
      */
-    if (! gh_string_p( text ) || ! gh_string_p( substr ))
+    if (! AG_SCM_STRING_P( text ) || ! AG_SCM_STRING_P( substr ))
         return scm_num_eq_p( text, substr );
 
     pzText   = ag_scm2zchars( text, "text to match" );
@@ -853,6 +853,7 @@ Select_Match_NonExistence( char* pzText, char* pzMatch )
  *
  *  what:   Select one of several template blocks
  *  handler_proc:
+ *  load_proc:
  *
  *  desc:
  *
@@ -918,6 +919,7 @@ Select_Match_NonExistence( char* pzText, char* pzMatch )
 /*=macfunc ESAC
  *
  *  what:   Terminate the @code{CASE} Template Block
+ *  in-context:
  *
  *  desc:
  *    This macro ends the @code{CASE} function template block.
@@ -991,7 +993,7 @@ mFunc_Case( tTemplate* pT, tMacro* pMac )
                          pzSampleText );
 
                 if (OPT_VALUE_TRACE < TRACE_EVERYTHING)
-                    fprintf( pfTrace, zFileLine, pCurTemplate->pzFileName,
+                    fprintf( pfTrace, zFileLine, pCurTemplate->pzTplFile,
                              pMac->lineNo );
             }
 
@@ -1016,7 +1018,7 @@ mFunc_Case( tTemplate* pT, tMacro* pMac )
                          pT->pzTemplText + pMac->ozText );
 
                 if (OPT_VALUE_TRACE < TRACE_EVERYTHING)
-                    fprintf( pfTrace, zFileLine, pCurTemplate->pzFileName,
+                    fprintf( pfTrace, zFileLine, pCurTemplate->pzTplFile,
                              pMac->lineNo );
             }
 
@@ -1139,7 +1141,10 @@ mLoad_Case( tTemplate* pT, tMacro* pMac, tCC** ppzScan )
 /*=macfunc SELECT
  *
  *  what:    Selection block for CASE function
+ *  in-context:
  *  alias:   | ~ | = | * | ! | + |
+ *  unload-proc:
+ *
  *  desc:
  *    This macro selects a block of text by matching an expression
  *    against the sample text expression evaluated in the @code{CASE}
@@ -1158,7 +1163,7 @@ mLoad_Select( tTemplate* pT, tMacro* pMac, tCC** ppzScan )
     long           srcLen = pMac->res;           /* macro len  */
 
     tSCC zInvSel[] = "Invalid selection clause";
-    teFuncType typ = FTYP_SELECT_COMPARE_FULL;
+    int  typ       = (int)FTYP_SELECT_COMPARE_FULL;
 
     /*
      *  Set the global macro loading mode
@@ -1175,13 +1180,13 @@ mLoad_Select( tTemplate* pT, tMacro* pMac, tCC** ppzScan )
     if (*pzSrc == '*') {
         pzSrc++;
         if (isspace( *pzSrc ) || (*pzSrc == NUL)) {
-            typ    = FTYP_SELECT_MATCH_ANYTHING;
+            typ    = (int)FTYP_SELECT_MATCH_ANYTHING;
             srcLen = 0;
             pMac->ozText = 0;
             goto selection_done;
         }
 
-        typ |= FTYP_SELECT_COMPARE_SKP_START;
+        typ |= (int)FTYP_SELECT_COMPARE_SKP_START;
     }
 
     /*
@@ -1193,7 +1198,7 @@ mLoad_Select( tTemplate* pT, tMacro* pMac, tCC** ppzScan )
         /*
          *  Or in the pattern matching bit
          */
-        typ |= FTYP_SELECT_MATCH_FULL;
+        typ |= (int)FTYP_SELECT_MATCH_FULL;
         pMac->res = REG_EXTENDED;
         /* FALLTHROUGH */
 
@@ -1206,7 +1211,7 @@ mLoad_Select( tTemplate* pT, tMacro* pMac, tCC** ppzScan )
         if (pzSrc[0] == pzSrc[-1]) {
             pzSrc++;
         } else {
-            typ |= FTYP_SELECT_EQUIVALENT_FULL;
+            typ |= (int)FTYP_SELECT_EQUIVALENT_FULL;
         }
         break;
 
@@ -1222,8 +1227,8 @@ mLoad_Select( tTemplate* pT, tMacro* pMac, tCC** ppzScan )
         if ((pzSrc[1] != NUL) && (! isspace( pzSrc[1] )))
             goto bad_sel;
 
-        typ = (pzSrc[-1] == '!')
-            ? FTYP_SELECT_MATCH_NONEXISTENCE : FTYP_SELECT_MATCH_EXISTENCE;
+        typ = (int)((pzSrc[-1] == '!')
+            ? FTYP_SELECT_MATCH_NONEXISTENCE : FTYP_SELECT_MATCH_EXISTENCE);
         srcLen = 0;
         pMac->ozText = 0;
         goto selection_done;
@@ -1240,7 +1245,7 @@ mLoad_Select( tTemplate* pT, tMacro* pMac, tCC** ppzScan )
      */
     if (*pzSrc == '*') {
         pzSrc++;
-        typ |= FTYP_SELECT_COMPARE_SKP_END;
+        typ |= (int)FTYP_SELECT_COMPARE_SKP_END;
     }
 
     if (! isspace( *pzSrc ))
@@ -1254,7 +1259,8 @@ mLoad_Select( tTemplate* pT, tMacro* pMac, tCC** ppzScan )
     /*
      *  See if we are doing case insensitive regular expressions
      */
-    if ((typ & FTYP_SELECT_EQV_MATCH_FULL) == FTYP_SELECT_EQV_MATCH_FULL) {
+    if (   (typ & (int)FTYP_SELECT_EQV_MATCH_FULL)
+        == (int)FTYP_SELECT_EQV_MATCH_FULL) {
         int  bitSet;
         pMac->res = REG_EXTENDED | REG_ICASE;
 
@@ -1262,9 +1268,9 @@ mLoad_Select( tTemplate* pT, tMacro* pMac, tCC** ppzScan )
          *  Turn off the case comparison mode for regular expressions.
          *  We don't have to worry about it.  It is done for us.
          */
-        bitSet  = ~FTYP_SELECT_EQUIVALENT_FULL;
-        bitSet |= FTYP_SELECT_COMPARE_FULL; /* dont turn this bit off! */
-        typ &= bitSet;
+        bitSet  = ~(int)FTYP_SELECT_EQUIVALENT_FULL;
+        bitSet |= (int)FTYP_SELECT_COMPARE_FULL; /* dont turn this bit off! */
+        typ    &= bitSet;
     }
 
     /*
@@ -1272,7 +1278,7 @@ mLoad_Select( tTemplate* pT, tMacro* pMac, tCC** ppzScan )
      */
     pzScan = pzCopy;
     pMac->ozText = (pzCopy - pT->pzTemplText);
-    if (typ == FTYP_SELECT_EQUIVALENT) {
+    if (typ == (int)FTYP_SELECT_EQUIVALENT) {
         do  {
             *(pzCopy++) = toupper( *(pzSrc++) );
         } while (--srcLen > 0);
@@ -1289,13 +1295,24 @@ mLoad_Select( tTemplate* pT, tMacro* pMac, tCC** ppzScan )
         spanQuote( (char*)pzScan );
 
  selection_done:
-    pMac->funcCode = typ;
+    pMac->funcCode = (teFuncType)typ;
 
     current_case.pSelect->sibIndex = (pMac - pT->aMacros);
     current_case.pSelect = (tMacro*)pMac;
 
     return pMac + 1;
 }
+
+void
+mUnload_Select( tMacro* pMac )
+{
+    if (pMac->funcPrivate != NULL) {
+        regex_t* pRe = (regex_t*)pMac->funcPrivate;
+        regfree( pRe );
+        AGFREE(pRe);
+    }
+}
+
 /*
  * Local Variables:
  * mode: C
