@@ -1,6 +1,6 @@
 
 /*
- *  $Id: funcIf.c,v 4.4 2005/06/07 22:26:43 bkorb Exp $
+ *  $Id: funcIf.c,v 4.8 2006/01/24 23:22:16 bkorb Exp $
  *
  *  This module implements the _IF text function.
  */
@@ -69,7 +69,7 @@ eval_true( void )
     char* pz = evalExpression( &needFree );
 
     if (isdigit( *pz ))
-        res = atoi( pz );
+        res = (atoi(pz) == 0) ? AG_FALSE : AG_TRUE;
 
     else switch (*pz) {
     case NUL:
@@ -106,6 +106,7 @@ eval_true( void )
  *  cindex:  conditional emit
  *  cindex:  if test
  *  handler_proc:
+ *  load_proc:
  *
  *  desc:
  *  Conditional block.  Its arguments are evaluated (@pxref{EXPR}) and
@@ -138,6 +139,7 @@ eval_true( void )
 /*=macfunc ENDIF
  *
  *  what:   Terminate the @code{IF} Template Block
+ *  in-context:
  *
  *  desc:
  *    This macro ends the @code{IF} function template block.
@@ -168,7 +170,7 @@ mFunc_If( tTemplate* pT, tMacro* pMac )
                          pMac->lineNo );
 
                 if (OPT_VALUE_TRACE < TRACE_EVERYTHING)
-                    fprintf( pfTrace, zFileLine, pCurTemplate->pzFileName,
+                    fprintf( pfTrace, zFileLine, pCurTemplate->pzTplFile,
                              pIf->lineNo );
             }
 
@@ -183,7 +185,7 @@ mFunc_If( tTemplate* pT, tMacro* pMac )
                  pCurTemplate->pzTemplText + pCurMacro->ozText );
 
         if (OPT_VALUE_TRACE < TRACE_EVERYTHING)
-            fprintf( pfTrace, zFileLine, pCurTemplate->pzFileName,
+            fprintf( pfTrace, zFileLine, pCurTemplate->pzTplFile,
                      pIf->lineNo );
     }
 
@@ -197,6 +199,7 @@ mFunc_If( tTemplate* pT, tMacro* pMac )
  *  cindex:  conditional emit
  *  cindex:  while test
  *  handler_proc:
+ *  load_proc:
  *
  *  desc:
  *  Conditionally repeated block.  Its arguments are evaluated (@pxref{EXPR})
@@ -220,6 +223,7 @@ mFunc_If( tTemplate* pT, tMacro* pMac )
 /*=macfunc ENDWHILE
  *
  *  what:   Terminate the @code{WHILE} Template Block
+ *  in-context:
  *
  *  desc:
  *    This macro ends the @code{WHILE} function template block.
@@ -234,7 +238,7 @@ mFunc_While( tTemplate* pT, tMacro* pMac )
     if (OPT_VALUE_TRACE >= TRACE_BLOCK_MACROS)
         fprintf( pfTrace, "WHILE `%s' loop in %s on line %d begins:\n",
                  pCurTemplate->pzTemplText + pCurMacro->ozText,
-                 pT->pzFileName, pMac->lineNo );
+                 pT->pzTplFile, pMac->lineNo );
 
     for (;;) {
         pCurTemplate = pT;
@@ -250,7 +254,7 @@ mFunc_While( tTemplate* pT, tMacro* pMac )
         fprintf( pfTrace, "WHILE macro repeated %d times\n", ct );
 
         if (OPT_VALUE_TRACE < TRACE_EVERYTHING)
-            fprintf( pfTrace, zFileLine, pT->pzFileName, pMac->lineNo );
+            fprintf( pfTrace, zFileLine, pT->pzTplFile, pMac->lineNo );
     }
 
     return pRet;
@@ -260,6 +264,7 @@ mFunc_While( tTemplate* pT, tMacro* pMac )
 /*=macfunc ELIF
  *
  *  what:   Alternate Conditional Template Block
+ *  in-context:
  *
  *  desc:
  *    This macro must only appear after an @code{IF} function, and
@@ -287,6 +292,7 @@ mLoad_Elif( tTemplate* pT, tMacro* pMac, tCC** ppzScan )
 /*=macfunc ELSE
  *
  *  what:   Alternate Template Block
+ *  in-context:
  *
  *  desc:
  *    This macro must only appear after an @code{IF} function,
@@ -471,11 +477,11 @@ ag_scm_set_writable( SCM set )
     switch (STATE_OPT( WRITABLE )) {
     case OPTST_DEFINED:
     case OPTST_PRESET:
-        fprintf( pfTrace, zWarn, pCurTemplate->pzFileName, pCurMacro->lineNo );
+        fprintf( pfTrace, zWarn, pCurTemplate->pzTplFile, pCurMacro->lineNo );
         break;
 
     default:
-        if (gh_boolean_p( set ) && (set == SCM_BOOL_F))
+        if (AG_SCM_BOOL_P( set ) && (set == SCM_BOOL_F))
             CLEAR_OPT( WRITABLE );
         else
             SET_OPT_WRITABLE;
